# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"remove_dsstore": "00_core.ipynb",
         "get_basename": "02_dataloader.ipynb",
         "read_image": "01_image.ipynb",
         "resize_image": "01_image.ipynb",
         "AUTOTUNE": "02_dataloader.ipynb",
         "show_batch": "02_dataloader.ipynb",
         "Clf": "02_dataloader.ipynb",
         "label_color": "03_visualization.ipynb",
         "colors": "03_visualization.ipynb",
         "draw_box": "03_visualization.ipynb",
         "draw_caption": "03_visualization.ipynb",
         "draw_boxes": "03_visualization.ipynb",
         "draw_detections": "03_visualization.ipynb",
         "draw_annotations": "03_visualization.ipynb",
         "limit_gpu": "04_utils.ipynb"}

modules = ["core.py",
           "image.py",
           "dataloader.py",
           "visualization.py",
           "utils.py"]

doc_url = "https://aniketmaurya.github.io/chitra/"

git_url = "https://github.com/aniketmaurya/chitra/tree/master/"

def custom_doc_links(name): return None
